from typing import Union
import json
from vka.base.buiders.button import InitializedButton


class Keyboard:
    """
    Генерирует клавиатуру на основе переданных кнопок.
    Чтобы добавить новый ряд, передайте Ellipsis (три точки)
    вместо кнопки. Клавиатуру можно создать и в другом стиле:
    вызывая `.add()` для каждой новой кнопки и `.new_line()`
    для перехода на следующий ряд
    Настройки `one_time` и `inline` передаются при самое инициализации
    Объект можно напрямую передать в поле `keyboard`
    при отправке сообщения
    """

    def __init__(
        self,
        *buttons: Union[InitializedButton, type(Ellipsis)],
        one_time: bool = True,
        inline: bool = False
    ) -> None:
        self.scheme = {"inline": inline, "buttons": [[]]}
        if not inline:
            self.scheme.update(one_time=one_time)

        self._build(*buttons)

    @staticmethod
    def empty() -> str:
        """
        Returns:
            Пустую клавиатуру. Используйте, чтобы удалить текущую
        """
        return '{"buttons":[],"one_time":true}'

    def add(self, button: InitializedButton) -> "Keyboard":
        """
        Добавляет в клавиатуру кнопку
        Arguments:
            button: Кнопка, которую надо добавить
        Returns:
            Текущая клавиатура
        """
        self.scheme["buttons"][-1].append(button.scheme)
        return self

    def new_line(self) -> "Keyboard":
        """
        Добавляет новый ряд клавиатуре
        """
        if not self.scheme["buttons"][-1]:
            raise ValueError("Can't add a new line if the last line is empty")
        self.scheme["buttons"].append([])
        return self

    def _build(
        self, *buttons: Union[InitializedButton, type(Ellipsis)]
    ) -> None:
        """
        Вспомогательный метод для построения рядов кнопок
        Arguments:
            buttons: Кнопки или Ellipsis (для новой линии)
        """
        for button in buttons:
            if button is Ellipsis:
                self.new_line()
            else:
                self.add(button)

    def __repr__(self) -> json.dumps:
        return json.dumps(self.scheme)





class User:

    def __init__(self, response):
        self.response = response

    @property
    def id(self):
        return self.response.id

    @property
    def fn(self):
        return self.response.first_name

    @property
    def ln(self):
        return self.response.last_name

    @property
    def full_name(self):
        return f"{self.fn} {self.ln}"

    def __format__(self, format_spec: str):
        format_spec = format_spec.replace(
            'fn', self.fn
        ).replace(
            'ln', self.ln
        ).replace(
            "full_name", self.full_name
        ).replace(
            'id', f"{self.id}"
        )

        if format_spec.startswith('@'):
            return f'@id{self.id} ({format_spec[1:]})'
        return format_spec



import asyncio
from typing import Any
from loguru import logger

from vka.base.longpoll import LongPoll
from vka.base.wrapper import Event
from vka.chatbot.checking import CheckingEventForCommand
from vka.chatbot.context import Context
import sys

sys.dont_write_bytecode = True


class ABot(LongPoll):

    """
    param debug:
        Включение debug режима
    param custom_event_name:
        Чтобы бот мог отловить другое событие
        [
            'wall_post_new', 'message_edit'
        ]
    param custom_event_func:
        ctx - обязательный параметр
        async def custom_event_func(ctx: Context): ...
    param custom_func:
        async def custom_func(bot: ABot): ...

    """

    def run(
            self, debug: bool = False,
            custom_event_name: list[str] = None,
            custom_event_func=None,
            custom_func=None
    ):
        try:
            asyncio.run(
                self.async_run(
                    debug=debug,
                    custom_event_name=custom_event_name,
                    custom_event_func=custom_event_func,
                    custom_func=custom_func,
                )
            )
        except (KeyboardInterrupt, SystemExit):
            return

    async def async_run(
            self, debug: bool = False,
            custom_event_name: list[str] = None,
            custom_event_func=None,
            custom_func=None,
    ):
        await self.async_init()
        self.custom_event_name = custom_event_name
        self.custom_event_func = custom_event_func
        if custom_func is not None:
            run_custom_func = asyncio.create_task(custom_func(self))
            self.set_item(key='run_custom_func', value=run_custom_func)
        await self._launching_bot(debug)

    async def _launching_bot(
            self, debug: bool,
    ):
        async for event in self.listen():
            if event.updates:
                asyncio.create_task(
                    self._wiretapping_type(
                        updates=event.updates, debug=debug
                    )
                )

    async def _wiretapping_type(
            self, updates, debug: bool,
    ):
        for i in updates:
            if debug:
                logger.opt(colors=True).debug(
                    f'[vka {self.group_id}] {i}'
                )
            await self._defining_events(update=i)

    async def _defining_events(self, update):
        """
        Определяем какой событие пришло от сервера
        """
        event = Event(update)
        ctx = Context(event=event, api=self.api, bot=self)
        check = CheckingEventForCommand(
            ctx=ctx,
            menu_commands=self.__menu_commands__,
            callback_action=self.__callback_action__,
            commands=self.__commands__
        )
        if self.custom_event_name is not None and update.type in self.custom_event_name:
            if self.custom_event_func is not None:
                return await self.custom_event_func(ctx)
        elif update.type == 'message_new':
            logger.opt(colors=True).info(
                f'[vka {self.group_id}] '
                f'<red>type: New message</red> '
                f'<white>peer_id: {ctx.msg.peer_id}</white> '
                f'<blue>from_id: '
                f'{ctx.msg.from_id}</blue> '
                f'<green>message: {ctx.msg.text}</green>'
            )
            if 'payload' in event.message:
                asyncio.create_task(
                    check.shipment_data_message_event(
                        obj=event.obj
                    )
                )
                return
            await check.search_for_command_message()

        elif update.type == 'message_event':
            logger.opt(colors=True).info(
                f'[vka {self.group_id}] '
                f'<red>type: Message event</red> '
                f'<white>peer_id: {ctx.msg.peer_id}</white> '
                f'<blue>from_id: '
                f'{ctx.msg.from_id}</blue> '
            )
            asyncio.create_task(
                check.shipment_data_message_event(
                    obj=event.obj,
                )
            )

    def add_command(
            self, *custom_filter,
            commands=(), any_text: bool = False,
            lvl: Any = None, show_snackbar: str = None,
            custom_answer: str = None
    ):
        """ используется декоратором над функцией """
        def wrapper(func):
            self.register_command(
                func=func, *custom_filter, commands=commands,
                any_text=any_text, lvl=lvl, show_snackbar=show_snackbar,
                custom_answer=custom_answer
            )
            return func

        return wrapper

    def register_command(
            self, func, *custom_filter,
            commands=(), any_text: bool = False,
            lvl: Any = None, show_snackbar: str = None,
            custom_answer: str = None,
    ):
        """
        Регистрирует команду в боте
        :param func: сама функция команды
        :param custom_filter: свой фильтр
        :param commands: команды | str / [str, ...]
        :param any_text: любой текст | bool
        :param lvl: дополнительный аргумент
        :param show_snackbar:
        :param custom_answer:
        :return:
        """
        self.__commands__.append(
            {
                'func_obj': func,
                'custom_filter': custom_filter,
                'any_text': any_text,
                'commands': commands,
                'lvl': lvl,
                'show_snackbar': show_snackbar,
                'custom_answer': custom_answer,
            }
        )

    def add_click_callback(
            self, *custom_filter,
            callback: bool = False,
            show_snackbar: bool = False,
    ):
        """
        используется декоратором над функцией

        :param callback:
        :param show_snackbar: исчезающее сообщение на экране

        в разработке:
            # :param open_link: открывает ссылку
            # :param open_app: открывает vk mini apps
        """

        def decorator(func):
            def wrapper():
                self.register_callback(
                    func=func, *custom_filter, callback=callback,
                    show_snackbar=show_snackbar
                )
                return func
            return wrapper()
        return decorator

    def register_callback(
            self,
            func, *custom_filter,
            callback: bool = False,
            show_snackbar: bool | str = False,
    ):
        """ можно использовать для добавлении кнопок без декораторов """
        self.__callback_action__[func.__name__] = {
            'func_obj': func,
            'custom_filter': custom_filter,
            'callback': callback,
            'show_snackbar': show_snackbar
        }




import asyncio
from traceback import format_exc
from typing import Optional
import aiohttp
from vka.api import version_api, API, ssl_context
from loguru import logger
from vka.base import AttrDict
from vka.base.wrapper import EventBase
from vka.chatbot import KeyAndBoxStorage


class LongPoll(KeyAndBoxStorage):
    """
    LongPoll
    """

    def __init__(
            self, token: str,
            wait: int = 25, lang: int = 0,
            version: str = version_api(),
    ):
        KeyAndBoxStorage.__init__(self)
        self.token = token
        self.lang = lang
        self.version = version
        self._wait = wait
        self.api: Optional[API] = None
        self.group_id = None
        self.request: Optional[aiohttp.ClientSession] = None

    async def async_init(self):
        self.request = aiohttp.ClientSession()
        self.api = API(token=self.token)
        await self.api.async_init()
        self.group_id = (
            await self.api.method('groups.getById', {})
        )[0].id
        await self._update_long_poll_server()

    async def _update_long_poll_server(self, ts: bool = True):
        long_poll = await self.api.groups.getLongPollServer(group_id=self.group_id)
        self._key = long_poll.key
        if ts:
            self._ts = long_poll.ts
        self._url = long_poll.server

    async def _check(self):
        data = {
            'act': 'a_check',
            'key': self._key,
            'ts': self._ts,
            'wait': self._wait,
        }
        response = await self.request.post(
            url=self._url, data=data, timeout=self._wait+10, ssl=ssl_context
        )
        response = AttrDict(await response.json())
        match response:
            case {'failed': failed} if failed == 1:
                self._ts = response.ts
            case {'failed': failed} if failed == 2:
                await self._update_long_poll_server(False)
            case {'failed': failed} if failed == 3:
                await self._update_long_poll_server()
            case _:
                self._ts = response.ts
                return EventBase(response)
        return await self._check()

    async def listen(self):
        try:
            logger.success(f"[vka] Запуск бота в группе -> @club{self.group_id}")
            while True:
                try:
                    yield await self._check()
                except asyncio.TimeoutError:
                    continue
                except Exception as vka_error:
                    logger.error(format_exc())
                    if str(vka_error) in 'Session is closed':
                        logger.success(f"[vka] Аварийная остановка бота -> @club{self.group_id}")
                        await self._lp_close()
                        return
                    await asyncio.sleep(1)
                    continue
        finally:
            logger.success(f"[vka] Остановка бота -> @club{self.group_id}")
            await self._lp_close()
            return

    async def _lp_close(self):
        await self.api.request.close()
        await self.request.close()
        self.__state__.clear()
        self.__message_ids__.clear()
        self.__commands__.clear()
        self.__addition__.clear()
        self.__menu_commands__.clear()
        self.__callback_action__.clear()
        self.get_item(key='run_custom_func').cancel()



import ssl
from datetime import datetime
import certifi
from aiohttp import ClientSession
from bs4 import BeautifulSoup

ssl_context = ssl.create_default_context(cafile=certifi.where())


class Registration:
    """
    Чтобы получить дату когда зарегистрирован пользователь
    """

    def __init__(
            self,
            user_id: int,
            request: ClientSession,
            url: str = 'https://vk.com/foaf.php?id=',
    ):
        self._url = url + str(user_id)
        self.request = request
        self.user_id = user_id

    async def soup(self) -> BeautifulSoup:
        response = await self.request.get(
            url=self._url, ssl=ssl_context
        )
        soup = BeautifulSoup(await response.text(), 'html.parser')
        return soup

    async def get_obj(self):
        s = await self.soup()

        last_logged_in = datetime.strptime(
            s.find('ya:lastloggedin').attrs.get('dc:date'),
            "%Y-%m-%dT%H:%M:%S+03:00"
        ) if s.find('ya:lastloggedin') is not None \
            else s.find('ya:lastloggedin')
        location = s.find('ya:location').attrs.get('ya:city') \
            if s.find('ya:location') is not None \
            else s.find('ya:location')

        public_access = s.find('ya:publicaccess').text \
            if s.find('ya:publicaccess') is not None else None

        profile_state = s.find('ya:profilestate').text \
            if s.find('ya:profilestate') is not None else None

        user_id = s.find('ya:uri').attrs.get('rdf:resource') \
            if s.find('ya:uri') is not None else None

        first_name = s.find('ya:firstname').text \
            if s.find('ya:firstname') is not None else None

        second_name = s.find('ya:secondname').text \
            if s.find('ya:secondname') is not None else None

        gender = s.find('foaf:gender').text \
            if s.find('foaf:gender') is not None else None

        subscribers_count = s.find('ya:subscriberscount').text \
            if s.find('ya:subscriberscount') is not None else None

        friends_count = s.find('ya:friendscount').text \
            if s.find('ya:friendscount') is not None else None

        subscribed_to_count = s.find('ya:subscribedtocount').text \
            if s.find('ya:subscribedtocount') is not None else None

        birthday = s.find('foaf:birthday').text \
            if s.find('foaf:birthday') is not None else None

        return {
            "public_access": public_access,
            "profile_state": profile_state,
            "id": user_id,
            "first_name": first_name,
            "second_name": second_name,
            "name": f"{first_name} {second_name}",
            "gender": gender,
            "subscribers_count": subscribers_count,
            "friends_count": friends_count,
            "subscribed_to_count": subscribed_to_count,
            "birthday": birthday,
            "location": location,
            "created": datetime.strptime(
                s.find('ya:created').attrs.get('dc:date'),
                "%Y-%m-%dT%H:%M:%S+03:00"
            ),
            "modified": datetime.strptime(
                s.find('ya:modified').attrs.get('dc:date'),
                "%Y-%m-%dT%H:%M:%S+03:00"
            ),
            "last_logged_in": last_logged_in,
        }



from vka.api import API, LANG, random_
from vka.chatbot.context import Context
from vka.bot import ABot
from vka.base.buiders.keyboard import Keyboard
from vka.base.buiders.button import Button
from vka.chatbot.wrappers.user import User
from vka.base.exception import VkApiError



# -*- coding: utf-8 -*-
from vka import Keyboard, Button

# TODO Переписать чтобы работало


def any_button(): ...
def back_button(): ...


class CreatorMenu:
    def __init__(self, menu_commands: list, command: str, func_obj):
        self.command = command
        self.func_obj = func_obj
        self.__menu_commands__ = menu_commands
        self.__menu_commands__.append({'command': command, 'func_obj': func_obj, 'button': []})
        self.index = [
            self.__menu_commands__.index(i) for i in self.__menu_commands__
            if i['command'] == self.command
        ][0]
        self.__menu_commands__[self.index]['menu'] = []

    def add_menu(
            self, command: str, icon: str,
            category: str = None, subcategory: str = None,
    ):
        self.__menu_commands__[self.index]['menu'].append(
            {
                'command': command,
                'category': category,
                'subcategory': subcategory,
                'icon': icon,
                'button': [],
            }
        )

    def keyboard_generation(
        self
    ):
        for i in self.__menu_commands__:
            if i['button']:
                keyboard = Keyboard(
                    inline=True
                )
                for j in self.__menu_commands__[self.index]['menu']:
                    if j['category'] is not None and \
                            j['category'] is None:
                        keyboard.add(
                            Button.callback(j['icon']).secondary().on_called(
                                self.func_obj, args=j['category']
                            ),
                        )
                if len(keyboard.scheme["buttons"]) == 3:
                    keyboard.new_line()

        for i in self.__menu_commands__[self.index]['menu']:
            if not i['button']:
                keyboard = Keyboard(
                    inline=True
                )
                for j in self.__menu_commands__[self.index]['menu']:
                    if i['category'] == j['subcategory'] and \
                            j['category'] is not None \
                            and i['category'] is not None:
                        keyboard.add(
                            Button.callback(j['icon']).secondary().on_called(
                                self.func_obj, args=j['category']
                            ),
                        )
                if len(keyboard.scheme["buttons"]) == 3:
                    keyboard.new_line()
                if keyboard.scheme["buttons"] != [[]]:
                    keyboard.add(
                        Button.callback('⬅️').secondary().on_called(
                            back_button, args=i['category']
                        ),
                    )
                i['button'] = keyboard if keyboard.scheme["buttons"] != [[]] else []


# m = CreatorMenu([], 'магазин', func_obj=any_button)
# m.add_menu(command='товары', category='store', icon='🧳')
# m.add_menu(command='история покупок', category='history_buy', icon='🛍')
# m.add_menu(command='активные покупки', category='active_buy', icon='🛒')
# m.add_menu(command='вип', category='vip', icon='💎', subcategory='store')
# m.add_menu(command='аренда', category='rent', icon='📃', subcategory='store')
# m.add_menu(
#     command='1 день - 1р\n7 дней - 10р\n30 дней - 50р',
#     icon='', subcategory='vip'
# )
# m.add_menu(
#     command='30 дней - 100р',
#     icon='', subcategory='rent'
# )
#
# m.keyboard_generation()




import inspect
import asyncio
import json

from loguru import logger

from vka.base import AttrDict
from vka.chatbot.context import Context


class CheckingEventForCommand:

    def __init__(
            self, ctx: Context,
            menu_commands: list,
            callback_action: dict,
            commands: list
    ):
        self._ctx = ctx
        self.menu_commands = menu_commands
        self.callback_action = callback_action
        self.commands = commands

    async def search_for_command_message(
            self
    ):
        await self.checking_command_in_menu()
        await self.checking_message_in_all_commands()

    async def checking_command_in_menu(self, obj=None, data: bool = False):
        if data:
            try:
                command = obj.payload.command
                argument = obj.payload.args
            except (KeyError, AttributeError):
                return
            for i in self.menu_commands:
                for j in i['menu']:
                    if j['category'] == argument or j['subcategory'] == argument:
                        await self.init_func(
                            i['func_obj'], self._ctx,
                            command=command, argument=i['button']
                        )
            return

        for i in self.menu_commands:
            cmd = self._ctx.msg.text.lower().split(' ')[
                  0:len(i['command'].split(' '))
                ]
            logger.debug(i['button'])
            if i['command'] in cmd:
                await self.init_func(
                    func=i['func_obj'],
                    ctx=self._ctx,
                    command=cmd,
                    argument=i['button']
                )

    async def checking_message_in_all_commands(self):
        """ Проверка на наличие команды во всех командах """
        for command in self.commands:
            cmd = ''
            match command['commands']:
                case _command if type(_command) is str:
                    cmd = self._ctx.msg.text.lower().split(' ')[
                          0:len(command['commands'].split(' '))
                          ]

            match command:
                case {'any_text': True}:
                    await self.init_func(
                        func=command['func_obj'],
                        ctx=self._ctx,
                        command=cmd,
                    )
                    continue
                case {'commands': _command} if type(_command) is tuple:
                    await self._checking_for_list(command)
                    continue
                case {'commands': _command} if _command in cmd:
                    self._ctx.command = _command
                    await self.init_func(
                        func=command['func_obj'],
                        ctx=self._ctx,
                        command=cmd,
                        argument=self._ctx.msg.text.lower().replace(
                            _command, ''
                        ).strip()
                    )
                    continue
                case {
                    'commands': _command
                      } if ''.join(_command) == self._ctx.msg.text:
                    await self.init_func(
                        func=command['func_obj'],
                        ctx=self._ctx,
                        command=cmd,
                        custom_answer=command['custom_answer']
                    )
                    continue

    async def shipment_data_message_event(self, obj):
        """
        Обрабатывает нажатие на кнопку
        """
        await self.checking_command_in_menu(obj=obj, data=True)
        event_data = {}
        try:
            command = obj.payload.command
            argument = obj.payload.args
        except (KeyError, AttributeError):
            try:
                argument = AttrDict(eval(obj.message.payload))
                command = eval(obj.message.payload)['command']
            except (KeyError, AttributeError):
                return
        saved_command = self.callback_action.get(command)
        if saved_command:
            if saved_command.get('click'):
                return await self.init_func(
                    saved_command['func_obj'], self._ctx,
                    command=command, argument=argument
                )
            if saved_command['callback']:
                await self.init_func(
                    saved_command['func_obj'], self._ctx,
                    command=command, argument=argument
                )
            if saved_command['show_snackbar']:
                argument = argument if argument != '' else None
                argument = argument if argument != () else None
                event_data['type'] = 'show_snackbar'
                event_data = await self._inspect_signature_executions(
                    self._ctx, saved_command['func_obj'], event_data,
                    argument=argument
                )
        else:
            gather = []
            for func in self.commands:
                if func['func_obj'].__name__ == command:
                    # ctx.cmd = func['commands'][-1] \
                    #     if type(func['commands']) is list \
                    #     else func['commands']

                    if func['show_snackbar'] is not None:
                        event_data['type'] = 'show_snackbar'
                        event_data['text'] = func['show_snackbar']
                        gather.append(
                            self._ctx.api.method(
                                'messages.sendMessageEventAnswer',
                                {
                                    "event_id": obj.event_id,
                                    "user_id": obj.user_id,
                                    "peer_id": obj.peer_id,
                                    "event_data": json.dumps(event_data)
                                }
                            )
                        )
                    gather.append(
                        self.init_func(
                            func['func_obj'], self._ctx,
                            command=command, argument=argument
                        )
                    )
                    await asyncio.gather(*gather)
                    return

        if event_data == {}:
            return
        return await self._ctx.api.method(
            'messages.sendMessageEventAnswer',
            {
                "event_id": obj.event_id,
                "user_id": obj.user_id,
                "peer_id": obj.peer_id,
                "event_data": json.dumps(event_data)
            }
        )

    @staticmethod
    async def _inspect_signature_executions(
            ctx: Context, func, event_data: dict, argument: str
    ):
        parameters = inspect.signature(
            func
        ).parameters
        if len(parameters) == 0:
            event_data['text'] = await func()
        elif len(parameters) == 1:
            event_data['text'] = await func(ctx)
        else:
            event_data['text'] = await func(ctx, argument)
        return event_data

    @staticmethod
    async def init_func(
            func,
            ctx: Context,
            command: str | list,
            argument: str = '',
            custom_answer: str = None
    ) -> asyncio.create_task:
        ctx.command = ''.join(command)
        if custom_answer is not None:
            return await ctx.answer(custom_answer)
        argument = argument if argument != '' else None
        argument = argument if argument != () else None
        sig = inspect.signature(func)

        parameters = [
            p for p in sig.parameters.keys()
            if p not in ['kwargs', 'args']
        ]

        if len(parameters) == 0 and argument is None:
            return asyncio.create_task(func())
        elif len(parameters) == 1 and argument is None:
            return asyncio.create_task(func(ctx))
        elif len(parameters) == 2:
            return asyncio.create_task(func(ctx, argument))
        else:
            return

    async def _checking_for_list(self, command: AttrDict):
        cmd = ''.join(
            [
                i
                for i in command['commands']
                if i in ' '.join(
                    self._ctx.msg.text.lower().split(' ')[0:len(i.split(' '))]
                )
            ]
        )
        if cmd in command['commands']:
            await self.init_func(
                func=command['func_obj'],
                ctx=self._ctx,
                command=cmd,
                argument=self._ctx.msg.text.replace(cmd, '').strip()
            )



class VkApiError(Exception):
    """
    Исключение, вызывается если VK API вернул ошибку в ответе
    """

    def __init__(self, error):
        self.error = error
        self.error_code = error.get("error_code")
        self.error_msg = error.get("error_msg")
        self.request_params = error.get("request_params")
        self.captcha_sid = error.get("captcha_sid")
        self.captcha_img = error.get("captcha_img")

    def __str__(self):
        return f"[{self.error_code}] {self.error_msg}"

    def __repr__(self):
        return f"[{self.error_code}] {self.error_msg}"


# class Captcha(VkApiError):
#     def __init__(self, ):


class VkaError(Exception): ...


import datetime
import json
import typing

from vka.base.attrdict import AttrDict
# from vka.base.wrapper import Event


def peer_id():
    return 2_000_000_000


class Message:

    def __init__(self, message):
        self.message = message

    @property
    def date(self) -> datetime:
        """ Время отправки сообщения """
        return datetime.datetime.fromtimestamp(self.message.date)

    @property
    def from_id(self) -> int:
        """ Идентификатор отправителя """
        return self.message.from_id

    @property
    def id(self) -> int:
        """ Идентификатор сообщения """
        return self.message.id

    @property
    def peer_id(self) -> int:
        """ Идентификатор назначения """
        return self.message.peer_id

    @property
    def text(self) -> str:
        """ Текст сообщения """
        return self.message.text

    @property
    def conversation_message_id(self) -> int:
        """ Уникальный автоматически увеличивающийся номер для всех сообщений с этим peer """
        return self.message.conversation_message_id

    @property
    def fwd_messages(self) -> list:
        """
            Массив пересланных сообщений (если есть).
            Максимальное количество элементов — 100.
            Максимальная глубина вложенности для пересланных сообщений — 45,
            общее максимальное количество в цепочке с учетом вложенности — 500.
        """
        return self.message.fwd_messages

    @property
    def important(self) -> bool:
        """ Сообщение помеченное как важное """
        return bool(self.message.important)

    @property
    def random_id(self) -> int:
        """
            Идентификатор, используемый при отправке сообщения.
            Возвращается только для исходящих сообщений
        """
        return self.message.random_id

    @property
    def attachments(self) -> list:
        """ Медиа-вложения сообщения (фотографии, ссылки и т.п.).  """
        return self.message.attachments

    @property
    def reply_message(self) -> AttrDict:
        """ Сообщение, в ответ на которое отправлено текущее """
        return self.message.reply_message

    @property
    def action(self) -> list:
        """ Информация о сервисном действии с чатом """
        return self.message.action

    @property
    def payload(self) -> AttrDict | None:
        """ Сервисное поле для сообщений ботам (полезная нагрузка) """
        if "payload" in self.message:
            return AttrDict(json.loads(self.message.payload))
        return None

    @property
    def is_hidden(self) -> bool:
        return bool(self.message.is_hidden)

    @property
    def chat_id(self) -> int:
        """ Идентификатор беседы """
        chat_id = self.peer_id - peer_id()
        if chat_id < 0:
            raise ValueError(
                "Can't get `chat_id` if message wasn't sent in a chat"
            )

        return chat_id

    @property
    def ref(self) -> typing.Any:
        """ Произвольный параметр для работы """
        return self.message.ref

    @property
    def ref_source(self) -> typing.Any:
        """ Произвольный параметр для работы """
        return self.message.ref_source



import asyncio
import json
from typing import Optional, List, Union, Dict, AsyncIterable
from loguru import logger
from vka import API, random_
from vka.base.wrapper import Event, Message
from vka.chatbot.wrappers.user import User


class Context:

    def __init__(self, event, api: API, bot):
        self._bot = bot
        self._api = api
        self._event = event
        self.command = ''

    @property
    def api(self) -> API:
        return self._api

    @property
    def event(self) -> Event:
        return self._event

    @property
    def msg(self) -> Message:
        return Message(self.event.obj)

    @property
    def bot(self) -> "ABot":
        return self._bot

    @property
    def group_id(self) -> int:
        return self.event.group_id

    async def receive_new_message(
            self,
            any_user: bool = False
    ) -> AsyncIterable["Context"]:
        """
        Метод для получение нового сообщение
        По умолчанию реагирует на того пользователя который написал команду.

        any_user: реагирует на любого человека

        Пример:
            >>> async for new_ctx in ctx.receive_new_message():
            >>>    await new_ctx.reply('привет')
        """
        async for new_event in self.receiving():
            if new_event.updates:
                for event in new_event.updates:
                    event = Event(event)
                    logger.success(event)
                    ctx = Context(event=event, api=self.api, bot=self)
                    if event.type in ['message_new', 'message_event']:
                        if any_user and ctx.msg.peer_id == self.msg.peer_id:
                            yield ctx
                        else:
                            if self.msg.from_id == ctx.msg.from_id \
                                    and ctx.msg.peer_id == self.msg.peer_id:
                                yield ctx

    async def answer(
            self,
            message: Optional[str] = None,
            *,
            random_id: Optional[int] = None,
            lat: Optional[float] = None,
            long: Optional[float] = None,
            attachment: Optional[List[Union[str]]] = None,
            reply_to: Optional[int] = None,
            forward_messages: Optional[List[int]] = None,
            sticker_id: Optional[int] = None,
            group_id: Optional[int] = None,
            keyboard: Optional[Union[str, json.dumps]] = None,
            payload: Optional[str] = None,
            dont_parse_links: Optional[int] = None,
            disable_mentions: bool = True,
            intent: Optional[str] = None,
            expire_ttl: Optional[int] = None,
            silent: Optional[bool] = None,
            subscribe_id: Optional[int] = None,
            content_source: Optional[str] = None,
            forward: Optional[str] = None,
            **kwargs
    ):
        if not kwargs.get('peer_id'):
            params = {'peer_id': self.msg.peer_id}
        else:
            params = {}
        return await self._messages_send(locals(), params)

    async def reply(
            self,
            message: Optional[str] = None,
            *,
            random_id: Optional[int] = None,
            lat: Optional[float] = None,
            long: Optional[float] = None,
            attachment: Optional[List[Union[str]]] = None,
            sticker_id: Optional[int] = None,
            group_id: Optional[int] = None,
            keyboard: Optional[Union[str, json.dumps]] = None,
            payload: Optional[str] = None,
            dont_parse_links: Optional[bool] = None,
            disable_mentions: bool = True,
            intent: Optional[str] = None,
            expire_ttl: Optional[int] = None,
            silent: Optional[bool] = None,
            subscribe_id: Optional[int] = None,
            content_source: Optional[str] = None,
            **kwargs,
    ):
        if not kwargs.get('peer_id'):
            params = {'peer_id': self.msg.peer_id}
        else:
            params = {}
        if self.msg.id:
            params["reply_to"] = self.msg.id
        else:
            params["forward"] = json.dumps({
                "is_reply": True,
                "conversation_message_ids": [
                    self.msg.conversation_message_id
                ],
                "peer_id": self.msg.peer_id,
            })
        return await self._messages_send(locals(), params)

    async def edit(
            self,
            message: str,
            conversation_message_id: int = None,
            # *,
            message_id: Optional[int] = None,
            lat: Optional[float] = None,
            long: Optional[float] = None,
            attachment: Optional[List[Union[str]]] = None,
            keep_forward_messages: bool = 1,
            keep_snippets: bool = 1,
            keyboard: Optional[Union[str, json.dumps]] = None,
            dont_parse_links: bool = 1,
            template: Optional[Union[str]] = None,
            expire_ttl: Optional[int] = None,
            **kwargs
    ):
        if not kwargs.get('peer_id'):
            params = {'peer_id': self.msg.peer_id}
        else:
            params = {}
        return await self._messages_edit(locals(), params)

    async def transmit(
            self,
            message: str,
            **kwargs,
    ):
        # TODO Переписать или удалить!
        """
        данная функция означает что будет изменено последнее сообщение бота
        """
        await self.edit(
            message,
            self.msg.conversation_message_id,
            **kwargs
        )

    async def fetch_sender(
            self,
            fields: List[str] = None,
            name_case: str = None,
            **kwargs
    ) -> list[User] | None | User:
        """
        Обертка чтобы чтобы получить информацию об сообщении о его отправители
        По умолчанию получает айди отправителя

        fields: подробное описание написано тут https://dev.vk.com/reference/objects/user

        name_case:
            • именительный – nom,
            • родительный – gen,
            • дательный – dat,
            • винительный – acc,
            • творительный – ins,
            • предложный – abl.

        kwargs:
            • replay=True: если на сообщение было отвечено;
            • fwd=True: если на сообщение было переслано;
            • all=True: все эти параметры сразу;

        Пример:
            >>> user = await ctx.fetch_sender()
            >>> await ctx.answer(f'Привет {user:fn}')

            • id              - id
            • fn              - имя
            • ln              - фамилия
            • full            - имя фамилия
            • @               - делает упоминание
        """
        if kwargs.get('reply'):
            try:
                user_ids = self.msg.reply_message.from_id
            except:
                return []
        elif kwargs.get('fwd'):
            try:
                user_ids = [fwd.from_id for fwd in self.msg.fwd_messages]
            except:
                return []
        elif kwargs.get('all'):
            user_ids = []
            try:
                user_ids.append(self.msg.reply_message.from_id)
            except: ...
            try:
                [
                    user_ids.append(user.from_id)
                    for user in self.msg.fwd_messages if user.from_id > 0
                ]
            except: ...
            user_ids.append(self.msg.from_id)
        else:
            user_ids = self.msg.from_id

        users_info = await self.api.method(
            "users.get", {
                "user_ids": user_ids,
                'fields': fields,
                'name_case': name_case
            }
        )
        if users_info == ():
            return []
        elif isinstance(user_ids, int):
            return User(users_info[0])
        elif len(users_info) == 1:
            return User(users_info[0])
        return [User(user) for user in users_info]

    async def user_get(
            self,
            user_ids: list | int | str,
            fields: List[str] = None,
            name_case: str = None
    ):
        """
        Обертка чтобы чтобы получить информацию об юзер айди, которые были переданы
        Про fields, name_case можно прочесть в методе fetch_sender
        """
        users_info = await self.api.method(
            "users.get", {
                "user_ids": user_ids,
                'fields': fields,
                'name_case': name_case
            }
        )
        if isinstance(user_ids, int) or isinstance(user_ids, str):
            try:
                return User(users_info[0])
            except IndexError:
                ...
        return [User(user) for user in users_info]

    def button_checking(
            self,
            func_name: object,
            any_user: [int, list, Ellipsis],
    ) -> bool:
        """
        Проверяем нажали ли на кнопку, если да то возвращает True иначе False
        :param any_user:
            1     - только если этим юзером
            [1,2] - только если этими юзерами
            ...   - всеми

        :param func_name: объект функции

        Пример:
            async for new_ctx in ctx.receive_new_message():
                if new_ctx.button_checking(obj, ...):
                    await new_ctx.answer('Ты нажал на кнопку')
        """
        if self.msg.payload is not None:
            command = eval(str(self.msg.payload))
            if func_name.__name__ == command['command']:
                if isinstance(any_user, int):
                    if self.msg.from_id == any_user:
                        return True
                elif isinstance(any_user, list):
                    if self.msg.from_id in any_user:
                        return True
                elif any_user is Ellipsis:
                    return True
            return False
        return False

    async def _messages_send(self, locals_: locals, params: Dict) -> dict:
        params = check_params(locals_=locals_, params=params)

        params['random_id'] = random_()
        messages_id = await self.api.method("messages.send", params)
        return messages_id

    async def _messages_edit(self, locals_: locals, params: Dict) -> dict:
        params = check_params(locals_=locals_, params=params)

        messages_id = await self.api.method("messages.edit", params)
        return messages_id

    async def receiving(self):
        while True:
            try:
                yield await self.bot._check()
            except asyncio.TimeoutError:
                continue
            except Exception as vka_error:
                logger.error(vka_error)
                continue

    async def for_receiving(self, new_event, any_user):
        for event in new_event:
            logger.warning(event)
            event = Event(event)
            ctx = Context(event=event, api=self.api, bot=self)

            match event.type:
                case ['message_new', 'message_event']:
                    if any_user:
                        yield ctx
                    else:
                        if self.msg.from_id in ctx.msg.from_id:
                            yield ctx


def check_params(locals_: locals, params: dict) -> dict:
    for name, value in locals_.items():
        if name == "kwargs":
            params.update(value)
        elif name != "self" and value is not None:
            params.update({name: value})

    del params["params"]
    return params



from __future__ import annotations
import json
import dataclasses
import functools
import typing


@dataclasses.dataclass
class _ButtonHandler:
    handler: typing.Callable


@dataclasses.dataclass
class ButtonOnclickHandler(_ButtonHandler):
    handler: typing.Callable[[...], typing.Awaitable]


@dataclasses.dataclass
class ButtonCallbackHandler(_ButtonHandler):
    handler: typing.Callable[[...], typing.Awaitable]


class InitializedButton:
    def __init__(self, **scheme) -> None:
        self.scheme = {"action": scheme}

    def _on_action(
        self, handler: _ButtonHandler, **kwargs
    ) -> InitializedButton:
        if self.scheme["action"].get("payload"):
            raise ValueError(
                "Payload has been already set. "
                "You can set only or onclick handler or "
                "custom payload, not bath at hte same time"
            )
        schema = dict(command=handler, args=kwargs)
        self.scheme["action"]["payload"] = json.dumps(schema)
        return self


# noinspection PyTypeChecker
class _ColoredButton(InitializedButton):
    def positive(self) -> _ColoredButton:
        """
        Зеленая кнопка (для обеих тем)
        """
        self.scheme["color"] = "positive"
        return self

    def negative(self) -> _ColoredButton:
        """
        Розовая (красная) кнопка (для обеих тем)
        """
        self.scheme["color"] = "negative"
        return self

    def primary(self) -> _ColoredButton:
        """
        Синяя кнопка для белой, белая для темной
        """
        self.scheme["color"] = "primary"
        return self

    def secondary(self) -> _ColoredButton:
        """
        Белая кнопка для светлой темы, серая для темной
        """
        self.scheme["color"] = "secondary"
        return self


class _ClickableColoredButton(_ColoredButton):
    def on_click(
        self, handler: ButtonOnclickHandler, **kwargs
    ) -> InitializedButton:
        return self._on_action(handler.__name__, **kwargs)


class _CallableColoredButton(_ColoredButton):
    # handler :ButtonCallbackHandler
    def on_called(self, handler: [ButtonCallbackHandler, typing.Any], **kwargs):
        return self._on_action(handler.__name__, **kwargs)


class _UncoloredButton(InitializedButton):
    def on_called(self, handler: [ButtonCallbackHandler, typing.Any], **kwargs):
        return self._on_action(handler.__name__, **kwargs)


def _convert_payload(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if "payload" in kwargs:
            if isinstance(kwargs["payload"], dict):
                kwargs["payload"] = json.dumps(
                    kwargs["payload"]
                )
            elif not isinstance(kwargs["payload"], str):
                raise TypeError(
                    "Invalid type for payload. "
                    "Payload can be only str, "
                    "dict or not passed"
                )
        return func(*args, **kwargs)

    return wrapper


class Button:
    @classmethod
    @_convert_payload
    def text(
        cls,
        label: str,
        *,
        payload: typing.Optional[typing.Union[str, dict]] = None,
    ) -> _ClickableColoredButton:
        """
        Кнопка типа `text`
        """
        return _ClickableColoredButton(
            label=label, type="text", payload=payload
        )

    @classmethod
    @_convert_payload
    def open_link(
        cls,
        label: str,
        *,
        link: str,
        payload: typing.Optional[typing.Union[str, dict]] = None,
    ) -> _UncoloredButton:
        """
        Кнопка типа `open_link`
        """
        return _UncoloredButton(
            label=label,
            link=link,
            type="open_link",
            payload=payload,
        )

    @classmethod
    @_convert_payload
    def location(
        cls, *, payload: typing.Optional[typing.Union[str, dict]] = None
    ) -> _UncoloredButton:
        """
        Кнопка типа `location`
        """
        return _UncoloredButton(type="location", payload=payload)

    @classmethod
    @_convert_payload
    def vk_pay(
        cls,
        *,
        hash_: str,
        payload: typing.Optional[typing.Union[str, dict]] = None,
    ) -> _UncoloredButton:
        """
        Кнопка типа `vkpay`
        """
        return _UncoloredButton(hash=hash_, type="vkpay", payload=payload)

    @classmethod
    @_convert_payload
    def open_app(
        cls,
        label: str,
        *,
        app_id: int,
        owner_id: int,
        hash_: str,
        payload: typing.Optional[typing.Union[str, dict]] = None,
    ) -> _UncoloredButton:
        """
        Кнопка типа `open_app`
        """
        return _UncoloredButton(
            label=label,
            app_id=app_id,
            owner_id=owner_id,
            hash=hash_,
            type="open_app",
            payload=payload,
        )

    @classmethod
    @_convert_payload
    def callback(
        cls,
        label: str,
        *,
        payload: typing.Optional[typing.Union[str, dict]] = None,
    ) -> _CallableColoredButton:
        """
        Кнопка типа `callback`
        """
        return _CallableColoredButton(
            label=label, type="callback", payload=payload
        )



import setuptools
with open(r'README.md', 'r', encoding='utf-8') as fh:
	long_description = fh.read()

setuptools.setup(
	name='vka',
	version='1.2.13',
	author='Major4ik',
	author_email='2772771882@mail.ru',
	description='module for the vk api wrapper',
	long_description=long_description,
	long_description_content_type='text/markdown',
	url='https://github.com/MrCreEper002/vka',
	include_package_data=True,
	packages=['vka', 'vka/base', 'vka/base/buiders', 'vka/chatbot', 'vka/chatbot/wrappers'],
	install_requires=[
		'loguru==0.6.0',
		'bs4==0.0.1',
	],
	classifiers=[
		'Programming Language :: Python :: 3.10',
		'Programming Language :: Python :: 3.11',
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	],
	python_requires='>=3.10',
)


# -*- coding: utf-8 -*-
from typing import Callable

from vka.base.exception import VkaError


class CreatorButtonCommands:
    """
    commands_button = {
        'menu': {
            'commands': [],  # команды от которых будет вызываться данная функция
            'text': ['...', ...],  # текст который будет выдаваться при написании команды или нажатие на кнопку
            'button': ['commands', ...],  # какие кнопки будут прикреплены кнопки к этому сообщению
            'attachment': ['', ...],  # вложение какие могут прикрепляться
            'function': ...,  # функция которая будет запускаться
        },
        'commands': {
            'text': '...',
        }
    }
    """

    def __init__(self):
        self.commands_button = {}

    def add(
            self, name_button: str, function: Callable,
            text: list[str], commands: list[str] = None,
            button: list[str] = None, attachment: list[str] = None,
    ):
        if self.commands_button.get(name_button):
            raise VkaError('Данная переменная уже добавлена.')

        self.commands_button[name_button] = {
            'function': function,
            'commands': commands,
            'text': text,
            'button': button,
            'attachment': attachment,
        }


def menu_button(): ...


creator_menu = CreatorButtonCommands()

creator_menu.add(name_button='menu', function=menu_button, text=['MENU'], button=['commands'])
creator_menu.add(name_button='commands', function=menu_button, text=['Commands'])

print(creator_menu.commands_button)



import typing
import datetime
from vka.base.attrdict import AttrDict


def peer_id():
    return 2_000_000_000


class EventBase:
    def __init__(self, event):
        self._event = event
        self._ts: str = event.ts
        self._updates: list = event.updates

    @property
    def ts(self) -> str:
        return self._ts

    @property
    def updates(self) -> list['Event']:
        return self._updates

    def __repr__(self):
        return f"{self._event}"


class Event:
    def __init__(self, event):
        self._event = event

    @property
    def type(self) -> str:
        return self._event.type

    @property
    def obj(self) -> AttrDict:
        return self._event.object

    @property
    def message(self) -> "Message":
        return self.obj.message

    @property
    def group_id(self) -> int:
        return self._event.group_id

    @property
    def event_id(self) -> str:
        return self._event.event_id

    def __repr__(self):
        return f'{self._event}'


class Wrapper(dict):

    def __init__(self, fields: AttrDict):
        super().__init__()
        if isinstance(fields, dict):
            fields = AttrDict(fields)
        self._fields = fields

    @property
    def fields(self):
        if 'message' in self._fields:
            return self._fields.message
        return self._fields


class Message(Wrapper):

    @property
    def date(self) -> datetime:
        """ Время отправки сообщения """
        return datetime.datetime.fromtimestamp(self.fields.date)

    @property
    def from_id(self) -> int:
        """ Идентификатор отправителя """
        try:
            return self.fields.from_id
        except AttributeError:
            return self.fields.user_id

    @property
    def id(self) -> int:
        """ Идентификатор сообщения """
        return self.fields.id

    @property
    def peer_id(self) -> int:
        """ Идентификатор назначения """
        return self.fields.peer_id

    @property
    def text(self) -> str:
        """ Текст сообщения """
        return self.fields.text

    @property
    def conversation_message_id(self) -> int:
        """ Уникальный автоматически увеличивающийся номер для всех сообщений с этим peer """
        return self.fields.conversation_message_id

    @property
    def fwd_messages(self) -> list:
        """
            Массив пересланных сообщений (если есть).
            Максимальное количество элементов — 100.
            Максимальная глубина вложенности для пересланных сообщений — 45,
            общее максимальное количество в цепочке с учетом вложенности — 500.
        """
        return self.fields.fwd_messages

    @property
    def important(self) -> bool:
        """ Сообщение помеченное как важное """
        return bool(self.fields.important)

    @property
    def random_id(self) -> int:
        """
            Идентификатор, используемый при отправке сообщения.
            Возвращается только для исходящих сообщений
        """
        return self.fields.random_id

    @property
    def attachments(self) -> list:
        """ Медиа-вложения сообщения (фотографии, ссылки и т.п.).  """
        return self.fields.attachments

    @property
    def reply_message(self) -> ['Message', None]:
        """ Сообщение, в ответ на которое отправлено текущее """
        if "reply_message" in self.fields:
            return self.__class__(self.fields.reply_message)
        return None

    @property
    def action(self) -> AttrDict:
        """ Информация о сервисном действии с чатом """
        return AttrDict(self.fields.action)

    @property
    def payload(self) -> AttrDict | None:
        """ Сервисное поле для сообщений ботам (полезная нагрузка) """
        if "payload" in self.fields:
            return AttrDict(self.fields.payload)
        return None

    @property
    def is_hidden(self) -> bool:
        return bool(self.fields.is_hidden)

    @property
    def chat_id(self) -> int | bool:
        """ Идентификатор беседы """
        chat_id = self.peer_id - peer_id()
        if chat_id < 0:
            return False
        return chat_id

    @property
    def ref(self) -> typing.Any:
        """ Произвольный параметр для работы """
        return self.fields.ref

    @property
    def ref_source(self) -> typing.Any:
        """ Произвольный параметр для работы """
        return self.fields.ref_source

    def __repr__(self):
        return f"{self.fields}"



import asyncio
import enum
import json
import random
import ssl
from typing import Union, Optional, Any
import aiohttp
import certifi
from loguru import logger

from vka.base import AttrDict
from vka.base.exception import VkApiError
from vka.chatbot.registration import Registration

ssl_context = ssl.create_default_context(cafile=certifi.where())


def random_() -> random:
    return random.getrandbits(31) * random.choice([-1, 1])


def version_api():
    return "5.131"


class LANG(enum.Enum):
    """
    LANG.RU.value
    """
    RU: int = 0  # русский
    UK: int = 1  # украинский
    BE: int = 2  # белорусский
    EN: int = 3  # английский
    ES: int = 4  # испанский
    FI: int = 5  # финский
    DE: int = 6  # немецкий
    IT: int = 7  # итальянский


class API:

    """
    Пример использование класса API:

    async def test_api():
        api = API(token='token')
        # сперва нужно создать сессию
        await api.async_init()

        # Есть два варианта обращение к API методам отличие лишь в написание
        # первый вариант
        users_get = await api.method('users.get', {'user_ids': 1})
        # второй вариант
        users_get = api.users.get(user_ids=1)

        # не забываем закрыть сессию
        await api.close()

    asyncio.run(test_api)
    """

    def __init__(
            self,
            *,
            token: str,
            version: Union[float, str] = version_api(),
            url: str = "https://api.vk.com/method/",
            lang: [LANG, int] = LANG.RU.value,
            proxy: str = None,
    ) -> None:
        self._url = url
        self._token = token
        self._method_name = ""
        self._version = version
        self._lang = lang,
        self.proxy = proxy
        self.request: Optional[aiohttp.ClientSession] = None

    async def async_init(self) -> None:
        """ Создание сессии """
        self.request = aiohttp.ClientSession()

    def __getattr__(self, attribute: str):
        if self._method_name:
            self._method_name += f".{attribute}"
        else:
            self._method_name = attribute
        return self

    async def __call__(self, *args, **request_params):
        return await self.method(self._method_name, request_params)

    async def method(self, method_name: str, params: dict):
        """ Обращение к API методу вк """
        self._method_name = ''
        params["access_token"] = self._token
        params["v"] = self._version
        params['lang'] = self._lang
        params = _convert_params_for_api(params)
        response = await self.request.post(
            self._url + method_name,
            data=params,
            proxy=self.proxy,
            ssl=ssl_context
        )
        response = AttrDict((await response.json()))
        match response:
            case {'error': error}:
                error = VkApiError(error)
                if error.error_code == 6:
                    await asyncio.sleep(0.5)
                    return await self.method(method_name, params)
                logger.error(response)
                raise error
            case {'response': _r}:
                return response.response
            case _:
                return response

    async def register(self, user_id) -> Registration:
        return Registration(user_id=user_id, request=self.request)

    async def close(self) -> None:
        await self.request.close()


def _convert_param_value(value: Any, /) -> Any:
    """
    Конвертирует параметр API запроса в соответствии
    с особенностями API и дополнительными удобствами

    value: Текущее значение параметра

    """
    # Для всех перечислений функция вызывается рекурсивно.
    # Массивы в запросе распознаются вк только если записать их как строку,
    # перечисляя значения через запятую
    if isinstance(value, (list, set, tuple)):
        updated_sequence = map(_convert_param_value, value)
        return ",".join(updated_sequence)

    # Все словари, как списки, нужно dumps в JSON
    elif isinstance(value, dict):
        return json.dumps(value)

    # Особенности `aiohttp`
    elif isinstance(value, bool):
        return int(value)

    else:
        return str(value)


def _convert_params_for_api(params: dict, /) -> dict:
    """
    Конвертирует словарь из параметров для метода API,
    учитывая определенные особенности

    params: Параметры, передаваемые для вызова метода API

    вывод:
        Новые параметры, которые можно передать
        в запрос и получить ожидаемый результат

    """
    updated_params = {
        (key[:-1] if key.endswith("_") else key): _convert_param_value(value)
        for key, value in params.items()
        if value is not None
    }
    return updated_params




class KeyAndBoxStorage:
    """
    KeyAndBoxStorage - это хранилище ключей и хранилище добавленных команд в бота для
    быстрого сохранения и использования их процессе работы
    """

    def __init__(self):
        self.__state__: dict = {}
        self.__message_ids__: dict = {}
        self.__commands__: list = []
        self.__menu_commands__: list = []
        self.__addition__: dict = {}
        self.__callback_action__: dict = {}

    def set_item(self, key, value):
        self.__state__[key] = value

    def get_item(self, key):
        return self.__state__[key]

    def del_item(self, item):
        del self.__state__[item]



